<?php

/**
 * testimonials actions.
 *
 * @package    drspinksadmin
 * @subpackage testimonials
 * @author     Brian Raines
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class newsActions extends sfActions
{
    /**
    * Executes index action
    *
    * @param sfRequest $request A request object
    */
    public function executeIndex(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage News Items');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the news item that you wish to manage or <a href="/news/add" class="link add">click here</a> to add a new news item.  You can view the news item message by hovering your cursor over the row.');

        $c = new Criteria();
        $c->addAscendingOrderByColumn(NewsItemPeer::NEWS_ITEM_TYPE);
        $c->addDescendingOrderByColumn(NewsItemPeer::NEWS_ITEM_DATE);
        $c->addDescendingOrderByColumn(NewsItemPeer::TITLE);

        $this->news_items = NewsItemPeer::doSelect($c);
    }

    public function executeEdit(sfWebRequest $request)
    {
        if ($request->hasParameter('news_item_id'))
        {
            $this->news_item = NewsItemPeer::retrieveByPK($request->getParameter('news_item_id'));

            $this->getResponse()->setSlot('title', 'Edit News Item');
            $this->getResponse()->setSlot('content', 'User the form below to edit this news item. Please note that if this is an "Event", you will have the option to supply an "End Date".');
            if (!($this->news_item instanceof NewsItem))
            {
                $this->getUser()->setFlash('error', 'The news item you were attempting to edit could not be found.');
                $this->redirect('@news');
            }
            else {
                $this->form = new NewsItemForm($this->news_item);
            }
        }
        else
        {
            $this->getResponse()->setSlot('title', 'Add News Item');
        $this->getResponse()->setSlot('content', 'User the form below to add a news item. Please note that if this is an "Event", you will have the option to supply an "End Date".');
            $this->form = new NewsItemForm();
        }

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('news_item'));

            if ($this->form->isValid())
            {
                $news_item = $this->form->getObject();
                $values = $this->form->getValues();

                if ($news_item->getNewsItemId() == '')
                {
                    $news_item->setCreatedBy($this->getUser()->getAttribute('user_id'));
                }
                $news_item->setTitle((isset($values['title'])) ? $values['title'] : '');
                $news_item->setExternalUrl((isset($values['external_url'])) ? $values['external_url'] : '');
                $news_item->setNewsItemDate((isset($values['news_item_date'])) ? $values['news_item_date'] : '');
                $news_item->setEventEndDate((isset($values['event_end_date'])) ? $values['event_end_date'] : '');
                $news_item->setContent((isset($values['content'])) ? $values['content'] : '');
                $news_item->setIsActive((isset($values['is_active'])) ? $values['is_active'] : '');
                $news_item->setNewsItemType((isset($values['news_item_type'])) ? $values['news_item_type'] : '');
                $news_item->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $news_item->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" news item was successfully saved.', $news_item->getTitle()));
                    $this->redirect('@news');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this news item.');
                }
            }
            else
            {
                $this->getResponse()->setSlot('error', 'There was an error trying to save this news item.');
            }
        }
    }

    public function executeDelete(sfWebRequest $request)
    {
        if ($request->hasParameter('news_item_id'))
        {
            $news_item = NewsItemPeer::retrieveByPK($request->getParameter('news_item_id'));

            if ($news_item instanceof NewsItem)
            {
                $news_item->delete();
            }

            $this->getUser()->setFlash('success', sprintf('The news item "%s" was successfully deleted.', $news_item->getTitle()));
            $this->redirect('@news');
        }
    }
}
