<?php

/**
 * offices actions.
 *
 * @package    drspinksadmin
 * @subpackage testimonials
 * @author     Brian Raines
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class officesActions extends sfActions
{
    /**
    * Executes index action
    *
    * @param sfRequest $request A request object
    */
    public function executeIndex(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage Office Locations');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the office location that you wish to manage or <a href="/offices/add" class="link add">click here</a> to add a new office location.');

        $c = new Criteria();
        $c->addAscendingOrderByColumn(OfficePeer::ADDRESS_1);

        $this->offices = OfficePeer::doSelect($c);
    }

    public function executeEdit(sfWebRequest $request)
    {
        if ($request->hasParameter('office_id'))
        {
            $this->office = OfficePeer::retrieveByPK($request->getParameter('office_id'));

            $this->getResponse()->setSlot('title', 'Edit Office Location');
            $this->getResponse()->setSlot('content', 'User the form below to edit this office location.');
            if (!($this->office instanceof Office))
            {
                $this->getUser()->setFlash('error', 'The office location you were attempting to edit could not be found.');
                $this->redirect('@offices');
            }
            else {
                $this->form = new OfficeForm($this->office);
            }
        }
        else
        {
            $this->getResponse()->setSlot('title', 'Add Office Location');
            $this->getResponse()->setSlot('content', 'User the form below to add an office location.');
            $this->form = new OfficeForm();
        }

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('office'));

            if ($this->form->isValid())
            {
                $office = $this->form->getObject();
                $values = $this->form->getValues();

                if ($office->getOfficeId() == '')
                {
                    $office->setCreatedBy($this->getUser()->getAttribute('user_id'));
                }
                $office->setAddress1((isset($values['address_1'])) ? $values['address_1'] : '');
                $office->setAddress2((isset($values['address_2'])) ? $values['address_2'] : '');
                $office->setAddress3((isset($values['address_3'])) ? $values['address_3'] : '');
                $office->setCity((isset($values['city'])) ? $values['city'] : '');
                $office->setState((isset($values['state'])) ? $values['state'] : '');
                $office->setZip((isset($values['zip'])) ? $values['zip'] : '');
                $office->setPhone((isset($values['phone'])) ? $values['phone'] : '');
                $office->setFax((isset($values['fax'])) ? $values['fax'] : '');
                $office->setIsActive((isset($values['is_active']) && $values['is_active'] == 0) ? false : true);
                $office->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $office->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" office location was successfully saved.', $office->getAddress1()));
                    $this->redirect('@offices');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this office location.');
                }
            }
            else
            {
                $this->getResponse()->setSlot('error', 'There was an error trying to save this office location.');
            }
        }
    }

    public function executeDelete(sfWebRequest $request)
    {
        if ($request->hasParameter('office_id'))
        {
            $office = OfficePeer::retrieveByPK($request->getParameter('office_id'));

            if ($office instanceof Office)
            {
                $office->delete();
            }

            $this->getUser()->setFlash('success', sprintf('The office "%s" was successfully deleted.', $office->getAddress1()));
            $this->redirect('@offices');
        }
    }
}
