<?php

/**
 * pages actions.
 *
 * @package    admin
 * @subpackage pages
 * @author     Your name here
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class pagesActions extends sfActions
{
    /**
    * Executes index action
    *
    * @param sfRequest $request A request object
    */
    public function executeIndex(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage Pages');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the page that you wish to manage.');
        $this->pages = PagePeer::findAllPages();
    }
    /**
    * Executes edit action
    *
    * @param sfRequest $request A request object
    */
    public function executeEdit(sfWebRequest $request)
    {
        if ($request->hasParameter('page_id'))
        {
            $this->page = PagePeer::retrieveByPK($request->getParameter('page_id'));
            if (!($this->page instanceof Page))
            {
                $this->getUser()->setFlash('error', 'The page you were attempting to edit could not be found.');
                $this->redirect('pages/index');
            }
        }
        else
        {
            $this->getUser()->setFlash('error', 'The page you were attempting to edit could not be found.');
            $this->redirect('pages/index');
        }

        $this->getResponse()->setSlot('title', 'Edit Page: ' . $this->page->getTitle());
        $this->getResponse()->setSlot('content', 'User the form below to edit the content for this page.');

        $this->form = new PageForm($this->page);

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('page'));

            if ($this->form->isValid())
            {
                $page = $this->form->getObject();
                $values = $this->form->getValues();

                $page->setTitle((isset($values['title'])) ? $values['title'] : '');
                $page->setSubTitle((isset($values['sub_title'])) ? $values['sub_title'] : '');
                $page->setContent((isset($values['content'])) ? $values['content'] : '');
                $page->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $page->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" page was successfully saved.', $page->getTitle()));
                    $this->redirect('pages/index');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this page.');
                }
            }
        }
    }

    public function executeContentBlocks(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage Content Blocks');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the content block that you wish to manage.');

        $c = new Criteria();
        $this->content_blocks = ContentBlockPeer::doSelect($c);
    }

    public function executeEditContentBlock(sfWebRequest $request)
    {
        if ($request->hasParameter('code'))
        {
            $this->content_block = ContentBlockPeer::retrieveByPK($request->getParameter('code'));
            if (!($this->content_block instanceof ContentBlock))
            {
                $this->getUser()->setFlash('error', 'The content block you were attempting to edit could not be found.');
                $this->redirect('pages/contentBlocks');
            }
        }
        else
        {
            $this->getUser()->setFlash('error', 'The content block you were attempting to edit could not be found.');
            $this->redirect('pages/contentBlocks');
        }

        $this->getResponse()->setSlot('title', 'Edit Content Block: ' . $this->content_block->getCode());
        $this->getResponse()->setSlot('content', 'User the form below to edit the content for this content block.');

        $this->form = new ContentBlockForm($this->content_block);

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('content_block'));

            if ($this->form->isValid())
            {
                $content_block = $this->form->getObject();
                $values = $this->form->getValues();

                $content_block->setContent($values['content']);
                $content_block->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $content_block->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" content block was successfully saved.', $content_block->getCode()));

                    $this->redirect('pages/contentBlocks');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this content block.');
                }
            }
        }
    }

    public function executeArticleCategories(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage Patient Education Categories');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the category that you wish to manage.');
        $this->categories = ArticleCategoryPeer::getAllArticleCategories();
    }

    public function executeEditArticleCategory(sfWebRequest $request)
    {
        if ($request->hasParameter('article_category_id'))
        {
            $this->article_category = ArticleCategoryPeer::retrieveByPK($request->getParameter('article_category_id'));
            if (!($this->article_category instanceof ArticleCategory))
            {
                $this->getUser()->setFlash('error', 'The category you were attempting to edit could not be found.');
                $this->redirect('@article_categories');
            }
        }
        else
        {
            $this->getUser()->setFlash('error', 'The category you were attempting to edit could not be found.');
            $this->redirect('@article_categories');
        }

        $this->getResponse()->setSlot('title', 'Edit Category: ' . $this->article_category->getDisplayName());
        $this->getResponse()->setSlot('content', 'User the form below to edit the content for this category.');

        $this->form = new ArticleCategoryForm($this->article_category);

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('article_category'));

            if ($this->form->isValid())
            {
                $category = $this->form->getObject();
                $values = $this->form->getValues();

                $category->setDisplayName((isset($values['display_name'])) ? $values['display_name'] : '');
                $category->setContent((isset($values['content'])) ? $values['content'] : '');
                $category->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $category->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" category was successfully saved.', $category->getDisplayName()));
                    $this->redirect('@article_categories');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this category.');
                }
            }
        }
    }

    public function executeResume(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Upload a Curriculum Vitae');
        $this->getResponse()->setSlot('content', 'Click the "Choose File" button to upload your resume.  The website is expecting a file with a .pdf extension.');

        if ($request->isMethod('post'))
        {
            if ($_FILES["file"]["error"] > 0)
            {
                $this->getUser()->setFlash('error', 'There was an error trying to upload the resume: '.$_FILES["file"]["error"].'.');
            }
            else
            {
                $name_parts = explode(".", $_FILES["file"]["name"]);
                $format = array_pop($name_parts);

                move_uploaded_file($_FILES["file"]["tmp_name"], sfConfig::get('sf_data_dir').'/resume/resume.'.$format);

                $this->getUser()->setFlash('success', 'The resume was successfully uploaded.');
            }
            $this->redirect('@resume');
        }
    }
}
