<?php

/**
 * testimonials actions.
 *
 * @package    drspinksadmin
 * @subpackage testimonials
 * @author     Brian Raines
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class testimonialsActions extends sfActions
{
    /**
    * Executes index action
    *
    * @param sfRequest $request A request object
    */
    public function executeIndex(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', 'Manage Testimonials');
        $this->getResponse()->setSlot('content', 'Select the "Edit" link next to the testimonial that you wish to manage or <a href="/testimonials/add" class="link add">click here</a> to add a new testimonial.  You can view the testimonial message by hovering your cursor over the row.');

        if ($request->hasParameter('move') && $request->hasParameter('testimonial_id'))
        {
            $testimonial = TestimonialPeer::retrieveByPK($request->getParameter('testimonial_id'));
            $testimonial->move($request->getParameter('move'));

            $this->redirect('@testimonials');
        }

        $c = new Criteria();
        $c->addAscendingOrderByColumn(TestimonialPeer::DISPLAY_ORDER);

        $this->testimonials = TestimonialPeer::doSelect($c);
    }

    public function executeEdit(sfWebRequest $request)
    {
        if ($request->hasParameter('testimonial_id'))
        {
            $this->testimonial = TestimonialPeer::retrieveByPK($request->getParameter('testimonial_id'));

            $this->getResponse()->setSlot('title', 'Edit Testimonial');
            $this->getResponse()->setSlot('content', 'User the form below to edit this testimonial.');
            if (!($this->testimonial instanceof Testimonial))
            {
                $this->getUser()->setFlash('error', 'The testimonial you were attempting to edit could not be found.');
                $this->redirect('@testimonials');
            }
            else {
                $this->form = new TestimonialForm($this->testimonial);
            }
        }
        else
        {
            $this->getResponse()->setSlot('title', 'Add Testimonial');
            $this->form = new TestimonialForm();
        }

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('testimonial'));

            if ($this->form->isValid())
            {
                $testimonial = $this->form->getObject();
                $values = $this->form->getValues();

                if ($testimonial->getTestimonialId() == '')
                {
                    $testimonial->setDisplayOrder(TestimonialPeer::getNextDisplayOrder());
                    $testimonial->setCreatedBy($this->getUser()->getAttribute('user_id'));
                }
                $testimonial->setMessage((isset($values['message'])) ? $values['message'] : '');
                $testimonial->setClientName((isset($values['client_name'])) ? $values['client_name'] : '');
                $testimonial->setIsActive((isset($values['is_active'])) ? $values['is_active'] : '');
                $testimonial->setUpdatedBy($this->getUser()->getAttribute('user_id'));

                try
                {
                    $testimonial->save();

                    $this->getUser()->setFlash('success', sprintf('The "%s" testimonial was successfully saved.', $testimonial->getClientName()));
                    $this->redirect('@testimonials');
                }
                catch (Exception $e)
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this testimonial.');
                }
            }
            else
            {
                $this->getResponse()->setSlot('error', 'There was an error trying to save this testimonial.');
            }
        }
    }


    /**
    * Executes addTemplate action
    *
    * @param sfRequest $request A request object
    */
    public function executeDelete(sfWebRequest $request)
    {
        if ($request->hasParameter('testimonial_id'))
        {
            $testimonial = TestimonialPeer::retrieveByPK($request->getParameter('testimonial_id'));

            if ($testimonial instanceof Testimonial)
            {
                $testimonial->delete();
            }

            TestimonialPeer::reOrderTestimonials();

            $this->getUser()->setFlash('success', sprintf('The testimonial "%s" was successfully deleted.', $testimonial->getClientName()));
            $this->redirect('@testimonials');
        }
    }
}
