<?php
require_once sfConfig::get('sf_lib_dir').DIRECTORY_SEPARATOR.'vendor'.DIRECTORY_SEPARATOR.'recaptcha'.DIRECTORY_SEPARATOR.'recaptchalib.php';

/**
 * about actions.
 *
 * @package    drspinks
 * @subpackage about
 * @author     Brian Raines
 *
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class aboutActions extends sfActions
{
    public function preExecute()
    {
        $disabled_pages = sfConfig::get('app_disabled_pages');
        if (in_array($this->getModuleName().'|'.$this->getActionName(), $disabled_pages))
        {
            $this->redirect('@homepage');
        }

        $this->page = PagePeer::findByModuleAction($this->getModuleName(), $this->getActionName());
        if ($this->page instanceof Page)
        {
            $this->getResponse()->setSlot('title', $this->page->getTitle());
            $this->getResponse()->setSlot('sub_title', $this->page->getSubTitle());
            $this->getResponse()->setSlot('content', $this->page->getContent());
        }
    }

    public function executeDrSpinks(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('page_image', '/images/layout/SpinksOR.jpg');
    }

    public function executeTestimonials(sfWebRequest $request)
    {
        $this->recaptchaPublicKey = sfConfig::get('app_recaptcha_public_key');
        $recaptchaPrivateKey = sfConfig::get('app_recaptcha_private_key');

        $this->getResponse()->setSlot('page_image', '/images/audience.jpg');

        $c = new Criteria();
        $c->add(TestimonialPeer::IS_ACTIVE, TRUE);
        $c->addAscendingOrderByColumn(TestimonialPeer::DISPLAY_ORDER);
        $this->testimonials = TestimonialPeer::doSelect($c);

        $this->form = new TestimonialForm();

        if ($request->isMethod('post'))
        {
            $resp = recaptcha_check_answer($recaptchaPrivateKey,
                                          $_SERVER["REMOTE_ADDR"],
                                          $request->getParameter('recaptcha_challenge_field'),
                                          $request->getParameter('recaptcha_response_field'));

            if (!$resp->is_valid) {
              $this->getUser()->setFlash('error', 'The reCAPTCHA wasn\'t entered correctly. Please try it again.');
            } else {
                $testimonial = $request->getParameter('testimonial');
                $testimonial['is_active'] = false;

                $this->form->bind($testimonial);

                if ($this->form->isValid())
                {
                    $testimonial = $this->form->getObject();
                    $values = $this->form->getValues();

                    if ($testimonial->getTestimonialId() == '')
                    {
                        $testimonial->setDisplayOrder(TestimonialPeer::getNextDisplayOrder());
                    }
                    $testimonial->setMessage((isset($values['message'])) ? $values['message'] : '');
                    $testimonial->setClientName((isset($values['client_name'])) ? $values['client_name'] : '');
                    $testimonial->setIsActive(false);
                    $testimonial->setCreatedBy('webuser');
                    $testimonial->setUpdatedBy('webuser');

                    try
                    {
                        if ($testimonial->save())
                        {
                            $sns_config = sfConfig::get('app_amazon_sns');
                            $sns = new AmazonSNS(array(
                                'key' => sfConfig::get('app_amazon_key'),
                                'secret' => sfConfig::get('app_amazon_secret_key')
                            ));

                            $result = $sns->publish($sns_config['topic_arn'], sprintf($sns_config['message'], $testimonial->getClientName(), 'Testimonial', substr($testimonial->getMessage(), 0, 200).((strlen($testimonial->getMessage())>200) ? '...':'')));
                        }

                        $this->sendTestimonialEmail($testimonial);

                        $this->getUser()->setFlash('success', 'Thanks!! Your testimonial was successfully submitted.');
                        $this->redirect('@testimonials');


                    }
                    catch (Exception $e)
                    {
                        $this->getResponse()->setSlot('error', 'There was an error trying to save this testimonial.');
                    }
                }
                else
                {
                    $this->getResponse()->setSlot('error', 'There was an error trying to save this testimonial.');
                }
            }
        }
    }

    public function executeCurriculumVitae(sfWebRequest $request)
    {

    }

    public function executeDownloadCurriculumVitae(sfWebRequest $request)
    {
        header('Content-disposition: attachment; filename=DrTheodoreJSpinks.pdf');

        $this->setLayout(false);
        sfConfig::set('sf_web_debug', false);

        $pdfpath = sfConfig::get('sf_data_dir').'/resume/resume.pdf';

        // check if the file exists
        $this->forward404Unless(file_exists($pdfpath));

        // Adding the file to the Response object
        $this->getResponse()->clearHttpHeaders();
        $this->getResponse()->setHttpHeader('Pragma: public', true);
        $this->getResponse()->setContentType('application/pdf');
        $this->getResponse()->sendHttpHeaders();
        $this->getResponse()->setContent(readfile($pdfpath));

        return sfView::NONE;
    }

    protected function sendTestimonialEmail(Testimonial $testimonial)
    {
        $emails         = sfConfig::get('app_notification_contact');
        $ses_config     = sfConfig::get('app_amazon_ses');

        $ses = new AmazonSES(array(
            'key' => sfConfig::get('app_amazon_key'),
            'secret' => sfConfig::get('app_amazon_secret_key')
        ));

        $source = $ses_config['source'];

        $destination = array(
            'ToAddresses' => $emails
        );

        $azn_message = array(
            'Subject' => array(
                'Data' => 'SN> Testimonial from ' . $testimonial->getClientName()
            ),
            'Body' => array(
                'Html' => array(
                    'Data' => $this->getPartial('about/emailTestimonialHtml', array('testimonial' => $testimonial))
                ),
                'Text' => array(
                    'Data' => $this->getPartial('about/emailTestimonialText', array('testimonial' => $testimonial))
                )
            )
        );

        $opt = array(

        );

        $result = $ses->send_email($source, $destination, $azn_message, $opt);

        if ($result instanceof CFResponse)
        {
            if ($result->isOK())
            {

            }
        }
    }
}
