<?php
require_once sfConfig::get('sf_lib_dir').DIRECTORY_SEPARATOR.'vendor'.DIRECTORY_SEPARATOR.'recaptcha'.DIRECTORY_SEPARATOR.'recaptchalib.php';

/**
 * contact actions.
 *
 * @package    drspinks
 * @subpackage contact
 * @author     Brian Raines
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class contactActions extends sfActions
{
    public function preExecute()
    {
        $disabled_pages = sfConfig::get('app_disabled_pages');
        if (in_array($this->getModuleName().'|'.$this->getActionName(), $disabled_pages))
        {
            $this->redirect('@homepage');
        }

        $this->page = PagePeer::findByModuleAction($this->getModuleName(), $this->getActionName());
        if ($this->page instanceof Page)
        {
            $this->getResponse()->setSlot('title', $this->page->getTitle());
            $this->getResponse()->setSlot('sub_title', $this->page->getSubTitle());
            $this->getResponse()->setSlot('content', $this->page->getContent());
        }
    }

    /**
     * Executes index action
     *
     * @param sfRequest $request A request object
     */
    public function executeIndex(sfWebRequest $request)
    {
        $this->recaptchaPublicKey = sfConfig::get('app_recaptcha_public_key');
        $recaptchaPrivateKey = sfConfig::get('app_recaptcha_private_key');

        $this->form = new ContactUsForm();

        if ($request->isMethod('post'))
        {
            $resp = recaptcha_check_answer($recaptchaPrivateKey,
                                          $_SERVER["REMOTE_ADDR"],
                                          $request->getParameter('recaptcha_challenge_field'),
                                          $request->getParameter('recaptcha_response_field'));

            if (!$resp->is_valid) {
              $this->getUser()->setFlash('error', 'The reCAPTCHA wasn\'t entered correctly. Go back and try it again.');
            } else {
                $this->form->bind($request->getParameter('contact'));

                if ($this->form->isValid())
                {
                    $values = $this->form->getValues();

                    $message = new Message();
                    $message->setFirstName($values['first_name']);
                    $message->setLastName($values['last_name']);
                    $message->setInterestedIn($values['interested_in']);
                    $message->setPreferredResponse($values['preferred_response']);
                    $message->setEmail($values['email']);
                    $message->setPhone($values['phone']);
                    $message->setMessage($values['comments']);
                    if ($message->save())
                    {
                        $sns_config = sfConfig::get('app_amazon_sns');
                        $sns = new AmazonSNS(array(
                            'key' => sfConfig::get('app_amazon_key'),
                            'secret' => sfConfig::get('app_amazon_secret_key')
                        ));

                        $result = $sns->publish($sns_config['topic_arn'], sprintf($sns_config['message'], $message->getFirstName() . ' ' . $message->getLastName(), $message->getInterestedIn(), substr($message->getMessage(), 0, 200).((strlen($message->getMessage())>200) ? '...':'')));
                    }

                    $this->sendContactEmail($message);
                    $this->sendContactConfirmationEmail($message);

                    // @todo send thankyou email
                    $this->redirect('@contact_thanks');
                }
            }
        }
    }

    protected function sendContactEmail($message)
    {
        $emails         = sfConfig::get('app_notification_contact');
        $domain         = sfConfig::get('app_domain');
        $tracking_token = md5($message->getMessageId().sfConfig::get('app_salt'));
        $ses_config     = sfConfig::get('app_amazon_ses');

        $ses = new AmazonSES(array(
            'key' => sfConfig::get('app_amazon_key'),
            'secret' => sfConfig::get('app_amazon_secret_key')
        ));

        $source = $ses_config['source'];

        $destination = array(
            'ToAddresses' => $emails
        );

        $azn_message = array(
            'Subject' => array(
                'Data' => 'SN> Contact from ' . $message->getFirstName() . ' ' . $message->getLastName()
            ),
            'Body' => array(
                'Html' => array(
                    'Data' => $this->getPartial('contact/emailContactHtml', array('message' => $message, 'domain' => $domain, 'tracking_token' => $tracking_token))
                ),
                'Text' => array(
                    'Data' => $this->getPartial('contact/emailContactText', array('message' => $message))
                )
            )
        );

        $opt = array(

        );

        $result = $ses->send_email($source, $destination, $azn_message, $opt);

        if ($result instanceof CFResponse)
        {
            if ($result->isOK())
            {

            }
        }
    }

    protected function sendContactConfirmationEmail($message)
    {
        $ses = new AmazonSES(array(
            'key' => sfConfig::get('app_amazon_key'),
            'secret' => sfConfig::get('app_amazon_secret_key')
        ));

        $ses_config     = sfConfig::get('app_amazon_ses');
        $source = $ses_config['source'];

        $destination = array(
            'ToAddresses' => array($message->getEmail())
        );

        $azn_message = array(
            'Subject' => array(
                'Data' => 'Spinks Neurosurgery Contact'
            ),
            'Body' => array(
                'Html' => array(
                    'Data' => $this->getPartial('contact/emailContactConfirmationHtml', array('message' => $message))
                ),
                'Text' => array(
                    'Data' => $this->getPartial('contact/emailContactConfirmationText', array('message' => $message))
                )
            )
        );

        $opt = array(

        );

        $result = $ses->send_email($source, $destination, $azn_message, $opt);

        if ($result instanceof CFResponse)
        {
            if ($result->isOK())
            {

            }
        }
    }

    /**
     * Executes tracking action
     *
     * @param sfRequest $request A request object
     */
    public function executeTracking(sfWebRequest $request)
    {
        $message = MessagePeer::retrieveByPK($request->getParameter('message_id'));

        if ($message instanceof Message && $request->getParameter('token') == md5($message->getMessageId().sfConfig::get('app_salt')))
        {
            $message->setIsRead(1);
            $message->save();
        }

        $response = $this->getResponse();
        $response->setContentType('image/png');

        $image = file_get_contents(sfConfig::get('sf_web_dir') . "/images/layout/empty.png");
        $response->setContent($image);

        return sfView::NONE;
    }

    public function executeThanks(sfWebRequest $request)
    {

    }
}
