<?php

/**
 * Base project form.
 *
 * @package    drspinks
 * @subpackage contact us form
 * @author     Brian Raines
 */
class ContactUsForm extends BaseForm
{
    protected static $interested_in = array(
        'Appointment Question'          => 'Appointment Question',
        'Insurance Issue'               => 'Insurance Issue',
        'Treatment or Disease Question' => 'Treatment or Disease Question',
        'Vendor Contact or Question'    => 'Vendor Contact or Question',
        'Media Request'                 => 'Media Request',
        'Testimonial'                   => 'Testimonial',
        'Other'                         => 'Other',
    );

    protected static $preferred_response = array(
        'email' => 'Email',
        'phone' => 'Phone',
    );

    public function configure()
    {
        $this->setWidgets(array(
            'first_name'            => new sfWidgetFormInputText(),
            'last_name'             => new sfWidgetFormInputText(),
            'interested_in'         => new sfWidgetFormSelect(array('choices' => self::$interested_in)),
            'preferred_response'    => new sfWidgetFormSelect(array('choices' => self::$preferred_response)),
            'email'                 => new sfWidgetFormInputText(),
            'phone'                 => new sfWidgetFormInputText(),
            'comments'              => new sfWidgetFormTextarea(),
        ));

        $this->widgetSchema->setNameFormat('contact[%s]');

        $this->widgetSchema->setLabels(array(
            'first_name'            => 'First Name',
            'last_name'             => 'Last Name',
            'interested_in'         => 'Topic',
            'preferred_response'    => 'Preferred Response',
            'comments'              => 'Questions or Comments',
        ));

        $this->setValidators(array(
            'first_name'            => new sfValidatorString(array('max_length' => 45), array(
                'required'   => 'The first name is required.',
                'max_length' => 'The first name is too long. It must be less than %max_length% characters.',
            )),
            'last_name'             => new sfValidatorString(array('max_length' => 45), array(
                'required'   => 'The last name is required.',
                'max_length' => 'The last name is too long. It must be less than %max_length% characters.',
            )),
            'interested_in'         => new sfValidatorChoice(array('choices' => array_keys(self::$interested_in))),
            'preferred_response'    => new sfValidatorChoice(array('choices' => array_keys(self::$preferred_response))),
            'email'                 => new sfValidatorEmail(array('max_length' => 45), array(
                'required'   => 'The email is required.',
                'invalid'    => 'The Email address is invalid.',
                'max_length' => 'The email is too long. It must be less than %max_length% characters.',
            )),
            'phone'                 => new sfPhoneValidator(array(), array(
                'required'   => 'The phone is required.',
                'invalid'    => 'The phone number is invalid. Ex: 000-000-0000.',
            )),
            'comments'              => new sfValidatorString(array('max_length' => 300), array(
                'required'   => 'The comments field is required.',
                'max_length' => 'The comments are too long. It must be less than %max_length% characters.',
            )),
        ));

        parent::setup();
    }
}