<?php

/**
 * patients actions.
 *
 * @package    drspinks
 * @subpackage patients
 * @author     Brian Raines
 *
 * @version    SVN: $Id: actions.class.php 23810 2009-11-12 11:07:44Z Kris.Wallsmith $
 */
class patientsActions extends sfActions
{
    public function preExecute()
    {
        $disabled_pages = sfConfig::get('app_disabled_pages');
        if (in_array($this->getModuleName().'|'.$this->getActionName(), $disabled_pages))
        {
            $this->redirect('@homepage');
        }

        $this->page = PagePeer::findByModuleAction($this->getModuleName(), $this->getActionName());
        if ($this->page instanceof Page)
        {
            $this->getResponse()->setSlot('title', $this->page->getTitle());
            $this->getResponse()->setSlot('sub_title', $this->page->getSubTitle());
            $this->getResponse()->setSlot('content', $this->page->getContent());
        }
    }

    /**
     * Executes PatientEducation action
     *
     * @param sfRequest $request A request object
     */
    public function executePatientEducation(sfWebRequest $request)
    {
        $this->form = new ArticleSearchForm();
    }

    /**
     * Executes PatientEducation action
     *
     * @param sfRequest $request A request object
     */
    public function executePatientEducationCategory(sfWebRequest $request)
    {
        if ($request->hasParameter('category_code'))
        {
            $this->category = ArticleCategoryPeer::getByArticleCategoryCode($request->getParameter('category_code'));

            if ($this->category instanceof ArticleCategory)
            {
                $this->getResponse()->setSlot('title', $this->category->getDisplayName());
                $this->getResponse()->setSlot('content', $this->category->getContent());

                $this->sub_categories = ArticleCategoryPeer::getArticleCategories($this->category->getLeftId(), $this->category->getRightId());
                $this->articles = ArticlePeer::getArticles($this->category->getArticleCategoryId());
            }
            else
            {
                $this->redirect('@patient_education');
            }
        }
        else
        {
            $this->redirect('@patient_education');
        }
    }

    /**
     * Executes PatientEducation action
     *
     * @param sfRequest $request A request object
     */
    public function executePatientEducationArticle(sfWebRequest $request)
    {
        $this->getResponse()->setSlot('title', null);

        if ($request->hasParameter('category_code'))
        {
            $this->category = ArticleCategoryPeer::getByArticleCategoryCode($request->getParameter('category_code'));

            if (($this->category instanceof ArticleCategory) !== true)
            {
                $this->redirect('@patient_education');
            }

            if ($request->hasParameter('article_code'))
            {
                $this->article = ArticlePeer::getByArticleCode($request->getParameter('article_code'));
                if (!($this->article instanceof Article))
                {
                    $this->redirect('@patient_education_category?category_code='.$this->category->getArticleCategoryCode());
                }

            }
            else
            {
                $this->redirect('@patient_education_category?category_code='.$this->category->getArticleCategoryCode());
            }
        }
        else
        {
            $this->redirect('@patient_education');
        }

    }

    /**
     * Executes PatientEducationSearch action
     *
     * @param sfRequest $request A request object
     */
    public function executePatientEducationSearch(sfWebRequest $request)
    {
        $this->categories       = ArticleCategoryPeer::getArticleCategories();
        $this->sub_categories   = array();
        $this->articles         = array();
        $this->parent_category  = null;
        $this->category         = null;
        $this->keyword          = '';

        $this->form = new ArticleSearchForm();

        if ($request->isMethod('post'))
        {
            $this->form->bind($request->getParameter('article_search'));

            if ($this->form->isValid())
            {
                $values = $this->form->getValues();

                if (isset($values['keyword']) && $values['keyword'] != '')
                {
                    $this->getUser()->setAttribute('search_query', $values['keyword']);
                }
                else
                {
                    $this->getUser()->getAttributeHolder()->remove('search_query');
                }
            }
            else
            {
                $this->getUser()->getAttributeHolder()->remove('search_query');
            }
            $this->redirect('@patient_education_search');
        }


        $criteria   = new Criteria();

        $criteria->add(ArticlePeer::IS_ACTIVE, true);

        if ($this->getUser()->hasAttribute('search_query'))
        {
            $this->keyword = $this->getUser()->getAttribute('search_query');

            $criteria->add(ArticlePeer::KEYWORDS, '%'.$this->keyword.'%', Criteria::ILIKE);
            $criteria->addOr(ArticlePeer::TITLE, '%'.$this->keyword.'%', Criteria::ILIKE);
        }

        $criteria->addJoin(ArticlePeer::ARTICLE_CATEGORY_ID, ArticleCategoryPeer::ARTICLE_CATEGORY_ID);

        $criteria->addAscendingOrderByColumn(ArticleCategoryPeer::LEFT_ID);
        $criteria->addAscendingOrderByColumn(ArticlePeer::DISPLAY_ORDER);

        $this->pager = new sfPropelPager('Article', 500);
        $this->pager->setCriteria($criteria);
        $this->pager->setPage($request->getParameter('page', 1));
        $this->pager->init();
    }

    public function executePatientEducationArticlePrint(sfWebRequest $request)
    {
        if ($request->hasParameter('article_code'))
        {
            $this->article = ArticlePeer::getByArticleCode($request->getParameter('article_code'));
        }
    }
}
